from ipaddress import ip_network, ip_address


def get_client_ip(request):
    x_forwarded_for = request.META.get("HTTP_X_FORWARDED_FOR")
    if x_forwarded_for:
        ip = x_forwarded_for.split(",")[0]
    else:
        ip = request.META.get("REMOTE_ADDR")
    return ip


def ip_address_in_ip_range(client_ip, ip_ranges) -> bool:

    return any(
        ip_address(client_ip) in ip_network(ip_range, strict=False) for ip_range in ip_ranges
    )



from django.conf import settings
from django.http import HttpResponseForbidden

from ip_restrictor.utils import get_client_ip, ip_address_in_ip_range


class WhitelistIPMixin:
    """
    Mixin to restrict access CBV only to whitelisted IPs.
    """

    def dispatch(self, request, *args, **kwargs):

        ip_address = get_client_ip(request)
        allowed_ips = getattr(settings, "WHITELIST_IPS", [])

        if not ip_address_in_ip_range(ip_address, allowed_ips):
            return HttpResponseForbidden("Access denied: Your IP is not on allowed.")

        return super().dispatch(request, *args, **kwargs)


class BlackListIPMixin:
    """
    Mixin to restrict CBV access to all but blacklisted IPs.
    """

    def dispatch(self, request, *args, **kwargs):

        ip_address = get_client_ip(request)
        blocked_ips = getattr(settings, "WHITELIST_IPS", [])

        if ip_address_in_ip_range(ip_address, blocked_ips):
            return HttpResponseForbidden("Access denied: Your IP is not on allowed.")

        return super().dispatch(request, *args, **kwargs)



from django.conf import settings
from django.http import HttpResponseForbidden

from ip_restrictor.utils import get_client_ip, ip_address_in_ip_range


class IPRestrictMiddleware:
    """
    Middleware to apply global IP restrictions based on DEFAULT_IP_RESTRICTION_MODE in settings file.
    """

    def __init__(self, get_response):
        self.get_response = get_response
        self.default_mode = getattr(settings, "DEFAULT_IP_RESTRICTION_MODE", False)

    def __call__(self, request):
        allowed_ips = getattr(settings, "WHITELIST_IPS", [])
        blocked_ips = getattr(settings, "BLACKLIST_IPS", [])

        ip_address = get_client_ip(request)

        if self.default_mode == "WHITELIST" and not ip_address_in_ip_range(
            ip_address, allowed_ips
        ):
            return HttpResponseForbidden("Access denied: Your IP is not on allowed.")

        if self.default_mode == "BLACKLIST" and ip_address_in_ip_range(
            ip_address, blocked_ips
        ):
            return HttpResponseForbidden("Access denied: Your IP is not on allowed.")

        return self.get_response(request)



from django.http import HttpResponseForbidden
from django.conf import settings

from ip_restrictor.utils import get_client_ip, ip_address_in_ip_range


def allow_whitelist(view_func):
    """
    Decorator to restrict access only to whitelisted IPs.
    """

    def _wrapped_view(request, *args, **kwargs):
        ip_address = get_client_ip(request)
        allowed_ips = getattr(settings, "WHITELIST_IPS", [])

        if not ip_address_in_ip_range(ip_address, allowed_ips):
            return HttpResponseForbidden("Access denied: Your IP is not on allowed.")

        return view_func(request, *args, **kwargs)

    return _wrapped_view


def deny_blacklist(view_func):
    """
    Decorator to restrict access to all but blacklisted IPs.
    """

    def _wrapped_view(request, *args, **kwargs):
        ip_address = get_client_ip(request)
        blocked_ips = getattr(settings, "BLACKLIST_IPS", [])

        if ip_address_in_ip_range(ip_address, blocked_ips):
            return HttpResponseForbidden("Access denied: Your IP is not on allowed.")

        return view_func(request, *args, **kwargs)

    return _wrapped_view



WHITELIST_IPS = ["127.0.0.1", "192.168.1.100"]
BLACKLIST_IPS = ["10.0.0.5", "10.0.0.6"]

# Default restriction mode for all views
# Options: "whitelist", "blacklist", or False
DEFAULT_IP_RESTRICTION_MODE = False
